@@ -1,104 +1,137 @@
-#copyright ReportLab Inc. 2000-2001
+#Copyright ReportLab Europe Ltd. 2000-2004
 #see license.txt for license details
-#history http://cvs.sourceforge.net/cgi-bin/cvsweb.cgi/reportlab/graphics/renderPS.py?cvsroot=reportlab
-#$Header: /cvsroot/reportlab/reportlab/graphics/renderPS.py,v 1.23 2003/06/12 18:01:43 rgbecker Exp $
-__version__=''' $Id: renderPS.py,v 1.23 2003/06/12 18:01:43 rgbecker Exp $ '''
+#history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/graphics/renderPS.py
+__version__=''' $Id: renderPS.py 3082 2007-05-11 15:38:07Z rgbecker $ '''
 import string, types
-from reportlab.pdfbase.pdfmetrics import stringWidth # for font info
+from reportlab.pdfbase.pdfmetrics import getFont, stringWidth, unicode2T1 # for font info
 from reportlab.lib.utils import fp_str, getStringIO
 from reportlab.lib.colors import black
-from reportlab.graphics.renderbase import StateTracker, getStateDelta
+from reportlab.graphics.renderbase import Renderer, StateTracker, getStateDelta, renderScaledDrawing
 from reportlab.graphics.shapes import STATE_DEFAULTS
 import math
 from types import StringType
 from operator import getitem
 from reportlab import rl_config
+_ESCAPEDICT={}
+for c in xrange(256):
+    if c<32 or c>=127:
+        _ESCAPEDICT[chr(c)]= '\\%03o' % c
+    elif c in (ord('\\'),ord('('),ord(')')):
+        _ESCAPEDICT[chr(c)] = '\\'+chr(c)
+    else:
+        _ESCAPEDICT[chr(c)] = chr(c)
+    del c
 
+def _escape_and_limit(s):
+    R = []
+    aR = R.append
+    n = 0
+    for c in s:
+        c = _ESCAPEDICT[c]
+        aR(c)
+        n += len(c)
+        if n>=200:
+            n = 0
+            aR('\\\n')
+    return ''.join(R)
 
-# we need to create encoding vectors for each font we use, or they will
-# come out in Adobe's old StandardEncoding, which NOBODY uses.
-PS_WinAnsiEncoding="""
-/RE { %def
-  findfont begin
-  currentdict dup length dict begin
-    { %forall
-      1 index /FID ne { def } { pop pop } ifelse
-    } forall
-    /FontName exch def dup length 0 ne { %if
-      /Encoding Encoding 256 array copy def
-      0 exch { %forall
-        dup type /nametype eq { %ifelse
-          Encoding 2 index 2 index put
-          pop 1 add
-        }{ %else
-          exch pop
-        } ifelse
-      } forall
-    } if pop
-  currentdict dup end end
-  /FontName get exch definefont pop
-} bind def
-
-/WinAnsiEncoding [
-  39/quotesingle 96/grave 128/euro 130/quotesinglbase/florin/quotedblbase
-  /ellipsis/dagger/daggerdbl/circumflex/perthousand
-  /Scaron/guilsinglleft/OE 145/quoteleft/quoteright
-  /quotedblleft/quotedblright/bullet/endash/emdash
-  /tilde/trademark/scaron/guilsinglright/oe/dotlessi
-  159/Ydieresis 164/currency 166/brokenbar 168/dieresis/copyright
-  /ordfeminine 172/logicalnot 174/registered/macron/ring
-  177/plusminus/twosuperior/threesuperior/acute/mu
-  183/periodcentered/cedilla/onesuperior/ordmasculine
-  188/onequarter/onehalf/threequarters 192/Agrave/Aacute
-  /Acircumflex/Atilde/Adieresis/Aring/AE/Ccedilla
-  /Egrave/Eacute/Ecircumflex/Edieresis/Igrave/Iacute
-  /Icircumflex/Idieresis/Eth/Ntilde/Ograve/Oacute
-  /Ocircumflex/Otilde/Odieresis/multiply/Oslash
-  /Ugrave/Uacute/Ucircumflex/Udieresis/Yacute/Thorn
-  /germandbls/agrave/aacute/acircumflex/atilde/adieresis
-  /aring/ae/ccedilla/egrave/eacute/ecircumflex
-  /edieresis/igrave/iacute/icircumflex/idieresis
-  /eth/ntilde/ograve/oacute/ocircumflex/otilde
-  /odieresis/divide/oslash/ugrave/uacute/ucircumflex
-  /udieresis/yacute/thorn/ydieresis
-] def
-
+# we need to create encoding vectors for each font we use, or they will      
+ # come out in Adobe's old StandardEncoding, which NOBODY uses.      
+PS_WinAnsiEncoding="""       
+/RE { %def       
+  findfont begin     
+  currentdict dup length dict begin  
+ { %forall   
+   1 index /FID ne { def } { pop pop } ifelse    
+ } forall    
+ /FontName exch def dup length 0 ne { %if    
+   /Encoding Encoding 256 array copy def     
+   0 exch { %forall      
+     dup type /nametype eq { %ifelse     
+       Encoding 2 index 2 index put      
+       pop 1 add     
+     }{ %else    
+       exch pop      
+     } ifelse    
+   } forall      
+ } if pop    
+  currentdict dup end end    
+  /FontName get exch definefont pop  
+} bind def       
+ 
+/WinAnsiEncoding [       
+  39/quotesingle 96/grave 128/euro 130/quotesinglbase/florin/quotedblbase    
+  /ellipsis/dagger/daggerdbl/circumflex/perthousand  
+  /Scaron/guilsinglleft/OE 145/quoteleft/quoteright  
+  /quotedblleft/quotedblright/bullet/endash/emdash       
+  /tilde/trademark/scaron/guilsinglright/oe/dotlessi     
+  159/Ydieresis 164/currency 166/brokenbar 168/dieresis/copyright    
+  /ordfeminine 172/logicalnot 174/registered/macron/ring     
+  177/plusminus/twosuperior/threesuperior/acute/mu       
+  183/periodcentered/cedilla/onesuperior/ordmasculine    
+  188/onequarter/onehalf/threequarters 192/Agrave/Aacute     
+  /Acircumflex/Atilde/Adieresis/Aring/AE/Ccedilla    
+  /Egrave/Eacute/Ecircumflex/Edieresis/Igrave/Iacute     
+  /Icircumflex/Idieresis/Eth/Ntilde/Ograve/Oacute    
+  /Ocircumflex/Otilde/Odieresis/multiply/Oslash  
+  /Ugrave/Uacute/Ucircumflex/Udieresis/Yacute/Thorn  
+  /germandbls/agrave/aacute/acircumflex/atilde/adieresis     
+  /aring/ae/ccedilla/egrave/eacute/ecircumflex       
+  /edieresis/igrave/iacute/icircumflex/idieresis     
+  /eth/ntilde/ograve/oacute/ocircumflex/otilde       
+  /odieresis/divide/oslash/ugrave/uacute/ucircumflex     
+  /udieresis/yacute/thorn/ydieresis  
+] def    
 """
 
 class PSCanvas:
     def __init__(self,size=(300,300), PostScriptLevel=2):
         self.width, self.height = size
+        xtraState = []
+        self._xtraState_push = xtraState.append
+        self._xtraState_pop = xtraState.pop
+        self.comments = 0
         self.code = []
+        self.code_append = self.code.append
         self._sep = '\n'
         self._strokeColor = self._fillColor = self._lineWidth = \
             self._font = self._fontSize = self._lineCap = \
             self._lineJoin = self._color = None
 
-
-        self._fontsUsed = {}  # track them as we go
-
+        self._fontsUsed =   [] # track them as we go
         self.setFont(STATE_DEFAULTS['fontName'],STATE_DEFAULTS['fontSize'])
         self.setStrokeColor(STATE_DEFAULTS['strokeColor'])
         self.setLineCap(2)
         self.setLineJoin(0)
         self.setLineWidth(1)
-
         self.PostScriptLevel=PostScriptLevel
 
     def comment(self,msg):
-        self.code.append('%'+msg)
+        if self.comments: self.code_append('%'+msg)
 
     def drawImage(self, image, x1,y1, x2=None,y2=None): # Postscript Level2 version
         # select between postscript level 1 or level 2
-        if PostScriptLevel==1:
+        if self.PostScriptLevel==1:
             self._drawImageLevel1(image, x1,y1, x2=None,y2=None)
-        elif PostScriptLevel == 2 :
+        elif self.PostScriptLevel==2:
             self._drawImageLevel2(image, x1,y1, x2=None,y2=None)
         else :
-            raise 'PostScriptLevelException'
+            raise ValueError('Unsupported Postscript Level %s' % self.PostScriptLevel)
 
     def clear(self):
-        self.code.append('showpage') # ugh, this makes no sense oh well.
+        self.code_append('showpage') # ugh, this makes no sense oh well.
+
+    def _t1_re_encode(self):
+        if not self._fontsUsed: return
+        # for each font used, reencode the vectors   
+        C = []
+        for fontName in self._fontsUsed:     
+            fontObj = getFont(fontName)
+            if not fontObj._dynamicFont and fontObj.encName=='WinAnsiEncoding':
+                C.append('WinAnsiEncoding /%s /%s RE' % (fontName, fontName))    
+        if C:
+            C.insert(0,PS_WinAnsiEncoding)
+            self.code.insert(1, string.join(C, self._sep))
 
     def save(self,f=None):
         if not hasattr(f,'write'):
@@ -113,16 +146,9 @@ class PSCanvas:
 /m {moveto} bind def
 /l {lineto} bind def
 /c {curveto} bind def
+''' % (self.width,self.height))
 
-%s
-''' % (self.width,self.height, PS_WinAnsiEncoding))
-
-        # for each font used, reencode the vectors
-        fontReencode = []
-        for fontName in self._fontsUsed.keys():
-            fontReencode.append('WinAnsiEncoding /%s /%s RE' % (fontName, fontName))
-        self.code.insert(1, string.join(fontReencode, self._sep))
-
+        self._t1_re_encode()
         file.write(string.join(self.code,self._sep))
         if file is not f:
             file.close()
@@ -130,10 +156,12 @@ class PSCanvas:
             markfilename(f,creatorcode='XPR3',filetype='EPSF')
 
     def saveState(self):
-        self.code.append('gsave')
+        self._xtraState_push((self._fontCodeLoc,))
+        self.code_append('gsave')
 
     def restoreState(self):
-        self.code.append('grestore')
+        self.code_append('grestore')
+        self._fontCodeLoc, = self._xtraState_pop()
 
     def stringWidth(self, s, font=None, fontSize=None):
         """Return the logical width of the string if it were drawn
@@ -145,23 +173,23 @@ class PSCanvas:
     def setLineCap(self,v):
         if self._lineCap!=v:
             self._lineCap = v
-            self.code.append('%d setlinecap'%v)
+            self.code_append('%d setlinecap'%v)
 
     def setLineJoin(self,v):
         if self._lineJoin!=v:
             self._lineJoin = v
-            self.code.append('%d setlinejoin'%v)
+            self.code_append('%d setlinejoin'%v)
 
     def setDash(self, array=[], phase=0):
         """Two notations.  pass two numbers, or an array and phase"""
         # copied and modified from reportlab.canvas
         psoperation = "setdash"
         if type(array) == types.IntType or type(array) == types.FloatType:
-            self._code.append('[%s %s] 0 %s' % (array, phase, psoperation))
+            self.code_append('[%s %s] 0 %s' % (array, phase, psoperation))
         elif type(array) == types.ListType or type(array) == types.TupleType:
             assert phase >= 0, "phase is a length in user space"
             textarray = string.join(map(str, array))
-            self.code.append('[%s] %s %s' % (textarray, phase, psoperation))
+            self.code_append('[%s] %s %s' % (textarray, phase, psoperation))
 
     def setStrokeColor(self, color):
         self._strokeColor = color
@@ -172,9 +200,9 @@ class PSCanvas:
             self._color = color
             if color:
                 if hasattr(color, "cyan"):
-                    self.code.append('%s setcmykcolor' % fp_str(color.cyan, color.magenta, color.yellow, color.black))
+                    self.code_append('%s setcmykcolor' % fp_str(color.cyan, color.magenta, color.yellow, color.black))
                 else:
-                    self.code.append('%s setrgbcolor' % fp_str(color.red, color.green, color.blue))
+                    self.code_append('%s setrgbcolor' % fp_str(color.red, color.green, color.blue))
 
     def setFillColor(self, color):
         self._fillColor = color
@@ -183,69 +211,109 @@ class PSCanvas:
     def setLineWidth(self, width):
         if width != self._lineWidth:
             self._lineWidth = width
-            self.code.append('%s setlinewidth' % width)
+            self.code_append('%s setlinewidth' % width)
 
-    def setFont(self,font,fontSize):
-        self._fontsUsed[font] = 1
+    def setFont(self,font,fontSize,leading=None):
         if self._font!=font or self._fontSize!=fontSize:
-            self._font,self._fontSize = (font,fontSize)
-            self.code.append('(%s) findfont %s scalefont setfont' % (font,fp_str(fontSize)))
+            self._fontCodeLoc = len(self.code)
+            self._font = font
+            self._fontSize = fontSize
+            self.code_append('')
 
     def line(self, x1, y1, x2, y2):
         if self._strokeColor != None:
-            self.code.append('%s m %s l stroke' % (fp_str(x1, y1), fp_str(x2, y2)))
+            self.setColor(self._strokeColor)
+            self.code_append('%s m %s l stroke' % (fp_str(x1, y1), fp_str(x2, y2)))
 
     def _escape(self, s):
         '''
         return a copy of string s with special characters in postscript strings
         escaped with backslashes.
-        Have not handled characters that are converted normally in python strings
-        i.e. \n -> newline
         '''
-        str = string.replace(s, chr(0x5C), r'\\' )
-        str = string.replace(str, '(', '\(' )
-        str = string.replace(str, ')', '\)')
-        return str
+        try:
+            return _escape_and_limit(s)
+        except:
+            raise ValueError("cannot escape %s %s" % (s, repr(s)))
 
-    def drawString(self, s, x, y, angle=0):
+    def _issueT1String(self,fontObj,x,y,s):
+        fc = fontObj
+        code_append = self.code_append
+        fontSize = self._fontSize
+        fontsUsed = self._fontsUsed
+        escape = self._escape
+        if not isinstance(s,unicode):
+            try:
+                s = s.decode('utf8')
+            except UnicodeDecodeError,e:
+                i,j = e.args[2:4]
+                raise UnicodeDecodeError(*(e.args[:4]+('%s\n%s-->%s<--%s' % (e.args[4],s[i-10:i],s[i:j],s[j:j+10]),)))
+
+        for f, t in unicode2T1(s,[fontObj]+fontObj.substitutionFonts):
+            if f!=fc:
+                psName = f.face.name
+                code_append('(%s) findfont %s scalefont setfont' % (psName,fp_str(fontSize)))
+                if psName not in fontsUsed:
+                    fontsUsed.append(psName)
+                fc = f
+            code_append('%s m (%s) show ' % (fp_str(x,y),escape(t)))
+            x += f.stringWidth(t.decode(f.encName),fontSize)
+        if fontObj!=fc:
+            self._font = None
+            self.setFont(fontObj.face.name,fontSize)
+
+    def drawString(self, x, y, s, angle=0):
         if self._fillColor != None:
+            fontObj = getFont(self._font)
+            if not self.code[self._fontCodeLoc]:
+                psName = fontObj.face.name
+                self.code[self._fontCodeLoc]='(%s) findfont %s scalefont setfont' % (psName,fp_str(self._fontSize))
+                if psName not in self._fontsUsed:
+                    self._fontsUsed.append(psName)
             self.setColor(self._fillColor)
-            s = self._escape(s)
-## before inverting...
-##            if angle == 0 :   # do special case of angle = 0 first. Avoids a bunch of gsave/grestore ops
-##                self.code.append('%s m 1 -1 scale (%s) show 1 -1 scale' % (fp_str(x,y),s))
-##            else : # general case, rotated text
-##                self.code.append('gsave %s %s translate %s rotate' % (x,y,angle))
-##                self.code.append('0 0 m 1 -1 scale (%s) show' % s)
-##                self.code.append('grestore')
-            if angle == 0 :   # do special case of angle = 0 first. Avoids a bunch of gsave/grestore ops
-                self.code.append('%s m (%s) show ' % (fp_str(x,y),s))
-            else : # general case, rotated text
-                self.code.append('gsave %s %s translate %s rotate' % (x,y,angle))
-                self.code.append('0 0 m (%s) show' % s)
-                self.code.append('grestore')
+            if angle!=0:
+                self.code_append('gsave %s translate %s rotate' % (fp_str(x,y),fp_str(angle)))
+                x = y = 0
+            if fontObj._dynamicFont:
+                s = self._escape(s)
+                self.code_append('%s m (%s) show ' % (fp_str(x,y),s))
+            else:
+                self._issueT1String(fontObj,x,y,s)
+            if angle!=0:
+                self.code_append('grestore')
+
+    def drawCentredString(self, x, y, text, text_anchor='middle'):
+        if self._fillColor is not None:
+            textLen = stringWidth(text, self._font,self._fontSize)
+            if text_anchor=='end':
+                x -= textLen
+            elif text_anchor=='middle':
+                x -= textLen/2.
+            self.drawString(x,y,text)
+
+    def drawRightString(self, text, x, y):
+        self.drawCentredString(text,x,y,text_anchor='end')
 
     def drawCurve(self, x1, y1, x2, y2, x3, y3, x4, y4, closed=0):
         codeline = '%s m %s curveto'
         data = (fp_str(x1, y1), fp_str(x2, y2, x3, y3, x4, y4))
         if self._fillColor != None:
             self.setColor(self._fillColor)
-            self.code.append((codeline % data) + ' eofill')
+            self.code_append((codeline % data) + ' eofill')
         if self._strokeColor != None:
             self.setColor(self._strokeColor)
-            self.code.append((codeline % data)
+            self.code_append((codeline % data)
                             + ((closed and ' closepath') or '')
                             + ' stroke')
 
     ########################################################################################
 
-    def rect(self, x1,y1, x2,y2, rx=8, ry=8):
+    def rect(self, x1,y1, x2,y2, stroke=1, fill=1):
         "Draw a rectangle between x1,y1, and x2,y2"
         # Path is drawn in counter-clockwise direction"
 
         x1, x2 = min(x1,x2), max(x1, x2) # from piddle.py
         y1, y2 = min(y1,y2), max(y1, y2)
-        self.polygon(((x1,y1),(x2,y1),(x2,y2),(x1,y2)), closed=1)
+        self.polygon(((x1,y1),(x2,y1),(x2,y2),(x1,y2)), closed=1, stroke=stroke, fill = fill)
 
     def roundRect(self, x1,y1, x2,y2, rx=8, ry=8):
         """Draw a rounded rectangle between x1,y1, and x2,y2,
@@ -264,15 +332,16 @@ class PSCanvas:
 
         # choice between newpath and moveTo beginning of arc
         # go with newpath for precision, does this violate any assumptions in code???
-        rrCode = ['newpath'] # Round Rect code path
+        rr = ['newpath'] # Round Rect code path
+        a = rr.append
         # upper left corner ellipse is first
-        rrCode.append(ellipsePath % (x1+rx, y1+ry, rx, -ry, 90, 180))
-        rrCode.append(ellipsePath % (x1+rx, y2-ry, rx, -ry, 180, 270))
-        rrCode.append(ellipsePath % (x2-rx, y2-ry, rx, -ry, 270, 360))
-        rrCode.append(ellipsePath % (x2-rx, y1+ry, rx, -ry, 0,  90) )
-        rrCode.append('closepath')
+        a(ellipsePath % (x1+rx, y1+ry, rx, -ry, 90, 180))
+        a(ellipsePath % (x1+rx, y2-ry, rx, -ry, 180, 270))
+        a(ellipsePath % (x2-rx, y2-ry, rx, -ry, 270, 360))
+        a(ellipsePath % (x2-rx, y1+ry, rx, -ry, 0,  90) )
+        a('closepath')
 
-        self._fillAndStroke(rrCode)
+        self._fillAndStroke(rr)
 
     def ellipse(self, x1,y1, x2,y2):
         """Draw an orthogonal ellipse inscribed within the rectangle x1,y1,x2,y2.
@@ -305,12 +374,12 @@ class PSCanvas:
 
         if self._fillColor != None:
             self.setColor(self._fillColor)
-            self.code.append(codeline)
+            self.code_append(codeline)
             codelineAppended = 1
-            if self._strokeColor!=None: self.code.append('gsave')
+            if self._strokeColor!=None: self.code_append('gsave')
             self.lineTo(cx,cy)
-            self.code.append('eofill')
-            if self._strokeColor!=None: self.code.append('grestore')
+            self.code_append('eofill')
+            if self._strokeColor!=None: self.code_append('grestore')
 
         # stroke portion
         if self._strokeColor != None:
@@ -318,13 +387,13 @@ class PSCanvas:
             self.setColor(self._strokeColor)
             (startx, starty) = (cx+rx*math.cos(startAngleRadians), cy+ry*math.sin(startAngleRadians))
             if not codelineAppended:
-                self.code.append(codeline)
+                self.code_append(codeline)
             if fromcenter:
                 # move to center
                 self.lineTo(cx,cy)
                 self.lineTo(startx, starty)
-                self.code.append('closepath')
-            self.code.append('stroke')
+                self.code_append('closepath')
+            self.code_append('stroke')
 
     def _genArcCode(self, x1, y1, x2, y2, startAng, extent):
         "Calculate the path for an arc inscribed in rectangle defined by (x1,y1),(x2,y2)"
@@ -344,39 +413,40 @@ class PSCanvas:
 
         return codeline % data
 
-    def polygon(self, p, closed=0):
+    def polygon(self, p, closed=0, stroke=1, fill=1):
         assert len(p) >= 2, 'Polygon must have 2 or more points'
 
         start = p[0]
         p = p[1:]
 
-        polyCode = []
-        polyCode.append("%s m" % fp_str(start))
+        poly = []
+        a = poly.append
+        a("%s m" % fp_str(start))
         for point in p:
-            polyCode.append("%s l" % fp_str(point))
+            a("%s l" % fp_str(point))
         if closed:
-            polyCode.append("closepath")
+            a("closepath")
 
-        self._fillAndStroke(polyCode)
+        self._fillAndStroke(poly,stroke=stroke,fill=fill)
 
     def lines(self, lineList, color=None, width=None):
         if self._strokeColor != None:
             self._setColor(self._strokeColor)
             codeline = '%s m %s l stroke'
             for line in lineList:
-                self.code.append(codeline % (fp_str(line[0]),fp_str(line[1])))
+                self.code_append(codeline % (fp_str(line[0]),fp_str(line[1])))
 
     def moveTo(self,x,y):
-        self.code.append('%s m' % fp_str(x, y))
+        self.code_append('%s m' % fp_str(x, y))
 
     def lineTo(self,x,y):
-        self.code.append('%s l' % fp_str(x, y))
+        self.code_append('%s l' % fp_str(x, y))
 
     def curveTo(self,x1,y1,x2,y2,x3,y3):
-        self.code.append('%s c' % fp_str(x1,y1,x2,y2,x3,y3))
+        self.code_append('%s c' % fp_str(x1,y1,x2,y2,x3,y3))
 
     def closePath(self):
-        self.code.append('closepath')
+        self.code_append('closepath')
 
     def polyLine(self, p):
         assert len(p) >= 1, 'Polyline must have 1 or more points'
@@ -385,11 +455,11 @@ class PSCanvas:
             self.moveTo(p[0][0], p[0][1])
             for t in p[1:]:
                 self.lineTo(t[0], t[1])
-            self.code.append('stroke')
-
+            self.code_append('stroke')
 
     def drawFigure(self, partList, closed=0):
         figureCode = []
+        a = figureCode.append
         first = 1
 
         for part in partList:
@@ -399,56 +469,57 @@ class PSCanvas:
             if op == figureLine:
                 if first:
                     first = 0
-                    figureCode.append("%s m" % fp_str(args[:2]))
+                    a("%s m" % fp_str(args[:2]))
                 else:
-                    figureCode.append("%s l" % fp_str(args[:2]))
-                figureCode.append("%s l" % fp_str(args[2:]))
+                    a("%s l" % fp_str(args[:2]))
+                a("%s l" % fp_str(args[2:]))
 
             elif op == figureArc:
                 first = 0
                 x1,y1,x2,y2,startAngle,extent = args[:6]
-                figureCode.append(self._genArcCode(x1,y1,x2,y2,startAngle,extent))
+                a(self._genArcCode(x1,y1,x2,y2,startAngle,extent))
 
             elif op == figureCurve:
                 if first:
                     first = 0
-                    figureCode.append("%s m" % fp_str(args[:2]))
+                    a("%s m" % fp_str(args[:2]))
                 else:
-                    figureCode.append("%s l" % fp_str(args[:2]))
-                figureCode.append("%s curveto" % fp_str(args[2:]))
+                    a("%s l" % fp_str(args[:2]))
+                a("%s curveto" % fp_str(args[2:]))
             else:
                 raise TypeError, "unknown figure operator: "+op
 
         if closed:
-            figureCode.append("closepath")
+            a("closepath")
         self._fillAndStroke(figureCode)
 
-    def _fillAndStroke(self,code,clip=0):
-        if self._fillColor or self._strokeColor or clip:
+    def _fillAndStroke(self,code,clip=0,fill=1,stroke=1):
+        fill = self._fillColor and fill
+        stroke = self._strokeColor and stroke
+        if fill or stroke or clip:
             self.code.extend(code)
-            if self._fillColor:
-                if self._strokeColor or clip: self.code.append("gsave")
+            if fill:
+                if stroke or clip: self.code_append("gsave")
                 self.setColor(self._fillColor)
-                self.code.append("eofill")
-                if self._strokeColor or clip: self.code.append("grestore")
-            if self._strokeColor != None:
-                if clip: self.code.append("gsave")
+                self.code_append("eofill")
+                if stroke or clip: self.code_append("grestore")
+            if stroke:
+                if clip: self.code_append("gsave")
                 self.setColor(self._strokeColor)
-                self.code.append("stroke")
-                if clip: self.code.append("grestore")
+                self.code_append("stroke")
+                if clip: self.code_append("grestore")
             if clip:
-                self.code.append("clip")
-                self.code.append("newpath")
-
+                self.code_append("clip")
+                self.code_append("newpath")
 
     def translate(self,x,y):
-        self.code.append('%s translate' % fp_str(x,y))
+        self.code_append('%s translate' % fp_str(x,y))
 
     def scale(self,x,y):
-        self.code.append('%s scale' % fp_str(x,y))
+        self.code_append('%s scale' % fp_str(x,y))
 
     def transform(self,a,b,c,d,e,f):
-        self.code.append('[%s] concat' % fp_str(a,b,c,d,e,f))
+        self.code_append('[%s] concat' % fp_str(a,b,c,d,e,f))
 
     def _drawTimeResize(self,w,h):
         '''if this is used we're probably in the wrong world'''
@@ -463,7 +534,6 @@ class PSCanvas:
         calculated from image size. (x1,y1) is upper left of image, (x2,y2) is lower right of
         image in piddle coordinates."""
         # For now let's start with 24 bit RGB images (following piddlePDF again)
-        print "Trying to drawImage in piddlePS"
         component_depth = 8
         myimage = image.convert('RGB')
         imgwidth, imgheight = myimage.size
@@ -473,7 +543,7 @@ class PSCanvas:
             y2 = y1 + imgheight
         drawwidth = x2 - x1
         drawheight = y2 - y1
-        print 'Image size (%d, %d); Draw size (%d, %d)' % (imgwidth, imgheight, drawwidth, drawheight)
+        #print 'Image size (%d, %d); Draw size (%d, %d)' % (imgwidth, imgheight, drawwidth, drawheight)
         # now I need to tell postscript how big image is
 
         # "image operators assume that they receive sample data from
@@ -522,10 +592,10 @@ class PSCanvas:
 
         dataline = outstream.read(78)
         while dataline <> "":
-            self.code.append(dataline)
+            self.code_append(dataline)
             dataline= outstream.read(78)
-        self.code.append('% end of image data') # for clarity
-        self.code.append('grestore') # return coordinates to normal
+        self.code_append('% end of image data') # for clarity
+        self.code_append('grestore') # return coordinates to normal
 
     # end of drawImage
     def _AsciiHexEncode(self, input):  # also based on piddlePDF
@@ -536,16 +606,13 @@ class PSCanvas:
         return output.getvalue()
 
     def _drawImageLevel2(self, image, x1,y1, x2=None,y2=None): # Postscript Level2 version
-        Image = import_Image()
-        if not Image: return
+        '''At present we're handling only PIL'''
         ### what sort of image are we to draw
         if image.mode=='L' :
-            print 'found image.mode= L'
             imBitsPerComponent = 8
             imNumComponents = 1
             myimage = image
         elif image.mode == '1':
-            print 'found image.mode= 1'
             myimage = image.convert('L')
             imNumComponents = 1
             myimage = image
@@ -555,7 +622,6 @@ class PSCanvas:
             imBitsPerComponent = 8
 
         imwidth, imheight = myimage.size
-        # print 'imwidth = %s, imheight = %s' % myimage.size
         if not x2:
             x2 = imwidth + x1
         if not y2:
@@ -568,21 +634,20 @@ class PSCanvas:
             '%s %s scale' % (drawwidth,drawheight)])
 
         if imNumComponents == 3 :
-            self.code.append('/DeviceRGB setcolorspace')
+            self.code_append('/DeviceRGB setcolorspace')
         elif imNumComponents == 1 :
-            self.code.append('/DeviceGray setcolorspace')
-            print 'setting colorspace gray'
+            self.code_append('/DeviceGray setcolorspace')
         # create the image dictionary
-        self.code.append("""
+        self.code_append("""
 <<
 /ImageType 1
 /Width %d /Height %d  %% dimensions of source image
 /BitsPerComponent %d""" % (imwidth, imheight, imBitsPerComponent) )
 
         if imNumComponents == 1:
-            self.code.append('/Decode [0 1]')
+            self.code_append('/Decode [0 1]')
         if imNumComponents == 3:
-            self.code.append('/Decode [0 1 0 1 0 1]  %% decode color values normally')
+            self.code_append('/Decode [0 1 0 1 0 1]  %% decode color values normally')
 
         self.code.extend([  '/ImageMatrix [%s 0 0 %s 0 %s]' % (imwidth, -imheight, imheight),
                             '/DataSource currentfile /ASCIIHexDecode filter',
@@ -599,10 +664,10 @@ class PSCanvas:
 
         dataline = outstream.read(78)
         while dataline <> "":
-            self.code.append(dataline)
+            self.code_append(dataline)
             dataline= outstream.read(78)
-        self.code.append('> % end of image data') # > is EOD for hex encoded filterfor clarity
-        self.code.append('grestore') # return coordinates to normal
+        self.code_append('> % end of image data') # > is EOD for hex encoded filterfor clarity
+        self.code_append('grestore') # return coordinates to normal
 
 # renderpdf - draws them onto a canvas
 """Usage:
@@ -610,7 +675,6 @@ class PSCanvas:
     renderPS.draw(drawing, canvas, x, y)
 Execute the script to see some test drawings."""
 from shapes import *
-from renderbase import Renderer
 
 # hack so we only get warnings once each
 #warnOnce = WarnOnce()
@@ -619,7 +683,7 @@ from renderbase import Renderer
 def draw(drawing, canvas, x=0, y=0, showBoundary=rl_config.showBoundary):
     """As it says"""
     R = _PSRenderer()
-    R.draw(drawing, canvas, x, y, showBoundary=showBoundary)
+    R.draw(renderScaledDrawing(drawing), canvas, x, y, showBoundary=showBoundary)
 
 def _pointsFromList(L):
     '''
@@ -627,8 +691,9 @@ def _pointsFromList(L):
     produce a list of points [(x0,y0), (y1,y0),....]
     '''
     P=[]
-    for i in range(0,len(L),2):
-        P.append((L[i],L[i+1]))
+    a = P.append
+    for i in xrange(0,len(L),2):
+        a((L[i],L[i+1]))
     return P
 
 class _PSRenderer(Renderer):
@@ -639,27 +704,6 @@ class _PSRenderer(Renderer):
     def __init__(self):
         self._tracker = StateTracker()
 
-    def draw(self, drawing, canvas, x, y, showBoundary=rl_config.showBoundary):
-        """This is the top level function, which
-        draws the drawing at the given location.
-        The recursive part is handled by drawNode."""
-        #stash references for ease of  communication
-        self._canvas = canvas
-        canvas._drawing = self._drawing = drawing
-        try:
-            #bounding box
-            if showBoundary: canvas.rect(x, y, drawing.width, drawing.height)
-            canvas.saveState()
-            deltas = STATE_DEFAULTS.copy()
-            deltas['transform'] = [1,0,0,1,x,y]
-            self._tracker.push(deltas)
-            self.applyStateChanges(deltas, {})
-            self.drawNode(drawing)
-            self._tracker.pop()
-            canvas.restoreState()
-        finally:
-            del self._canvas, self._drawing, canvas._drawing
-
     def drawNode(self, node):
         """This is the recursive method called for each node
         in the tree"""
@@ -757,7 +801,7 @@ class _PSRenderer(Renderer):
                     x = x - textLen/2
                 else:
                     raise ValueError, 'bad value for text_anchor '+str(text_anchor)
-            self._canvas.drawString(text,x,y)
+            self._canvas.drawString(x,y,text)
 
     def drawPath(self, path):
         from reportlab.graphics.shapes import _renderPath
@@ -810,7 +854,19 @@ class _PSRenderer(Renderer):
                 fontsize = delta.get('fontSize', self._canvas._fontSize)
                 self._canvas.setFont(fontname, fontsize)
 
-def drawToFile(d,fn, showBoundary=rl_config.showBoundary):
+    def drawImage(self, image):
+        from reportlab.lib.utils import ImageReader
+        im = ImageReader(image.path)
+        x0 = image.x
+        y0 = image.y
+        x1 = image.width
+        if x1 is not None: x1 += x0
+        y1 = image.height
+        if y1 is not None: y1 += y0
+        self._canvas.drawImage(im._image,x0,y0,x1,y1)
+
+def drawToFile(d,fn, showBoundary=rl_config.showBoundary,**kwd):
+    d = renderScaledDrawing(d)
     c = PSCanvas((d.width,d.height))
     draw(d, c, 0, 0, showBoundary=showBoundary)
     c.save(fn)
